@@ -1,147 +1,94 @@
 package Apache;
-
 use strict;
-use mod_perl 1.17_01;
 use Exporter ();
 use Apache::Constants qw(OK DECLINED);
-use Apache::Connection ();
-use Apache::Server ();
-
-eval { require Apache::Table; };
 
 @Apache::EXPORT_OK = qw(exit warn);
+$Apache::VERSION = "1.18";
 
 *import = \&Exporter::import;
 
+unless(defined &bootstrap) {
+    require DynaLoader;
+    @Apache::ISA = qw(DynaLoader);
+}
+
 if (caller eq "CGI::Apache") {
     #we must die here outside of httpd so CGI::Switch works
-    die unless $ENV{MOD_PERL};
+    bootstrap Apache $Apache::VERSION;
+}
+else {
+    eval { bootstrap Apache $Apache::VERSION; } 
+}
+if($@) {
+    die "$@\n" if exists $ENV{MOD_PERL};
+    warn "warning: can't `bootstrap Apache $Apache::VERSION' outside of httpd\n";
 }
 
-{
-    no strict;
-    $VERSION = "1.27";
-    __PACKAGE__->mod_perl::boot($VERSION);
-}
-
-BEGIN {
-    *Apache::ReadConfig:: = \%ApacheReadConfig::;
-}
-
-sub httpd_conf {
-    shift;
-    push @Apache::ReadConfig::PerlConfig,
-      map "$_\n", @_;
+sub module {
+    my($self, $module) = @_;
+    eval "require $module;";
 }
 
 sub parse_args {
     my($wantarray,$string) = @_;
     return unless defined $string and $string;
     if(defined $wantarray and $wantarray) {
-	return map { Apache::unescape_url_info($_) } split /[=&;]/, $string, -1;
+	return map { Apache::unescape_url_info($_) } split /[=&]/, $string, -1;
     }
     $string;
 }
 
 sub content {
     my($r) = @_;
-    my $ct = $r->header_in("Content-type") || "";
-    return unless $ct =~ m!^application/x-www-form-urlencoded!;
+    my $ct = $r->header_in("Content-type");
+    return unless $ct eq "application/x-www-form-urlencoded";
     my $buff;
     $r->read($buff, $r->header_in("Content-length"));
-    return $buff unless wantarray;
-    parse_args(1, $buff);
+    parse_args(wantarray, $buff);
 }
 
 sub args {
-    my($r, $val) = @_;
-    my $args = @_ > 1 ? $r->query_string($val) : $r->query_string;
-    return $args unless wantarray;
-    parse_args(1, $args);
+    my($r) = @_;
+    parse_args(wantarray, $r->query_string);
 }
 
-*READ = \&read unless defined &READ;
+sub cgi_var {
+    my($r, $key) = @_;
+    my $val = $r->cgi_env($key);
+    return $val;
+}
 
+*READ = \&read;
 sub read {
-    my($r, $bufsiz, $offset) = @_[0,2,3];
-    my($nrd, $buf, $total);
-    $nrd = $total = 0;
-    $buf = "";
-    $_[1] = "" unless $offset;
-
-    $r->soft_timeout("Apache->read");
-
-    while($bufsiz) {
-	$nrd = $r->read_client_block($buf, $bufsiz) || 0;
-	if(defined $nrd and $nrd > 0) {
-	    $bufsiz -= $nrd;
-            if ($offset) {
-                substr($_[1], $offset) .= $buf;
-                #$_[1] .= $buf;
-            }
-            else {
-                $_[1] .= $buf;
-            }
-	    $total += $nrd;
-	    next if $bufsiz;
-	    last;
-	}
-	else {
-	    $_[1] = undef;
-	    last;
-	}
-    }
-    $r->kill_timeout;
-    return $total;
-}
-
-sub new_read {
     my($r, $bufsiz) = @_[0,2];
     my($nrd, $buf, $total);
     $nrd = $total = 0;
     $buf = "";
     $_[1] ||= "";
+    #$#{$_[1]} = $bufsiz unless defined $_[1];
 
-    if(my $rv = $r->setup_client_block) {
-	$r->log_error("Apache->read: setup_client_block returned $rv");
-	die $rv;
-    }
+    $r->hard_timeout("Apache->read");
 
-    #XXX: must set r->read_length to 0 here,
-    #since this read() method may be called in loop
-    #in which case, the second time in, should_client_block() 
-    #thinks we've already read the request body and returns 0
-    $r->read_length(0); 
-
-    unless($r->should_client_block) {
-	my $rl = $r->read_length;
-	$r->log_error("Apache->read: should_client_block returned 0 (rl=$rl)");
-	return 0;
-    }
-
-    $r->soft_timeout("Apache->read");
-    
     while($bufsiz) {
-	$nrd = $r->get_client_block($buf, $bufsiz) || 0;
+	$nrd = $r->read_client_block($buf, $bufsiz) || 0;
 	if(defined $nrd and $nrd > 0) {
 	    $bufsiz -= $nrd;
 	    $_[1] .= $buf;
  	    #substr($_[1], $total, $nrd) = $buf;
 	    $total += $nrd;
-	    $r->reset_timeout;
 	    next if $bufsiz;
 	    last;
 	}
 	else {
 	    $_[1] = undef;
-	    last;
 	}
     }
     $r->kill_timeout;
     return $total;
 }
 
-sub GETC { my $c; shift->READ($c,1); $c; }
+sub GETC { my $c; shift->read($c,1); $c; }
 
 #shouldn't use <STDIN> anyhow, but we'll be nice
 sub READLINE { 
@@ -151,49 +98,67 @@ sub READLINE {
     $line;
 }
 
+*PRINT = \&print;
+
 sub PRINTF {
     my $r = shift;
     my $fmt = shift;
     $r->print(sprintf($fmt, @_));
 }
-*printf = \&PRINTF;
-
-sub WRITE {
-    my($r, $buff, $length, $offset) = @_;
-    my $send = substr($buff, $offset, $length);
-    $r->print($send);
-}
 
 sub send_cgi_header {
     my($r, $headers) = @_;
     my $dlm = "\015?\012"; #a bit borrowed from LWP::UserAgent
+    my(@headerlines) = split /$dlm/, $headers;
     my($key, $val);
-    local $_;
-    while(($_, $headers) = split /$dlm/, $headers, 2) {
-	#warn "hunk=`$_'\n";
-	#warn "rest=`$headers'\n";
-	if ($_ && /^(\S+?):\s*(.*)$/) {
+
+    foreach (@headerlines) {
+	if (/^(\S+?):\s*(.*)$/) {
 	    ($key, $val) = ($1, $2);
 	    last unless $key;
 	    $r->cgi_header_out($key, $val);
 	}
 	else {
-	    #warn "mod_perl: found header terminator\n";
-	    my $not_sent = 0;
-	    if($Apache::__SendHeader) {
-		$not_sent = not $r->sent_header;
+	    #we didn't find the header terminator, punt away
+	    if(!$r->sent_header()) {
+		$r->send_http_header;
 	    }
-	    else {
-		$not_sent = 1;
-	    }
-            if ($not_sent) {
-                $r->send_http_header;
-                $r->sent_header(-2);
-            }
-	    $r->print($headers); #send rest of buffer, without stripping newlines!!!
-	    last;
+	    $r->print($_);
 	}
     }
+    if($headers =~ /${dlm}${dlm}$/ or
+       $headers =~ /^(${dlm}){1,2}$/) 
+    {
+        $r->send_http_header;
+    }
+}
+
+sub as_string {
+    my($r) = @_;
+    my($k,$v,@retval);
+    my(%headers_in) = $r->headers_in;
+
+    push @retval, $r->the_request;
+    while(($k,$v) = each %headers_in) {
+	push @retval, "$k: $v";
+    }
+
+    push @retval, "";
+
+    push @retval, $r->status_line;
+    for (qw(err_headers_out headers_out)) {
+	my(%headers_out) = $r->$_();
+
+	while(($k,$v) = each %headers_out) {
+	    push @retval, "$k: $v";
+	}
+    }    
+    join "\n", @retval, "";
+}
+
+sub TIEHANDLE {
+    my($class, $r) = @_;
+    $r ||= Apache->request;
 }
 
 1;
@@ -219,9 +184,9 @@ the description of the Apache C API at http://www.apache.org/docs/.
 
 The request object holds all the information that the server needs to
 service a request.  Apache B<Perl*Handler>s will be given a reference to the
-request object as parameter and may choose to update or use it in various
+request object as parameter and may choose update or use it in various
 ways.  Most of the methods described below obtain information from or
-update the request object.
+updates the request object.
 The perl version of the request object will be blessed into the B<Apache> 
 package, it is really a C<request_rec*> in disguise.
 
@@ -260,11 +225,6 @@ structure or C<undef> if there is no previous request.
 This method returns a blessed reference to the next (internal) request
 structure or C<undef> if there is no next request.
 
-=item $r->last
-
-This method returns a blessed reference to the last (internal) request
-structure.  Handy for logging modules.
-
 =item $r->is_main
 
 Returns true if the current request object is for the main request.
@@ -342,7 +302,7 @@ The number of bytes sent to the client, handy for logging, etc.
 
 =item $r->the_request
 
-The request line sent by the client, handy for logging, etc.
+The request line send by the client, handy for logging, etc.
 
 =item $r->proxyreq
 
@@ -361,19 +321,10 @@ The $r->protocol method will return a string identifying the protocol
 that the client speaks.  Typical values will be "HTTP/1.0" or
 "HTTP/1.1".
 
-=item $r->hostname
-
-Returns the server host name, as set by full URI or Host: header.
-
-=item $r->request_time
-
-Returns the time that the request was made.  The time is the local unix
-time in seconds since the epoch.
-
 =item $r->uri( [$uri] )
 
-The $r->uri method will return the requested URI minus optional query
-string, optionally changing it with the first argument.
+The $r->uri method will return the requested URI, optionally changing
+it with the first argument.
 
 =item $r->filename( [$filename] )
 
@@ -387,7 +338,7 @@ The $r->path_info method will return what is left in the path after the
 I<URI --E<gt> filename> translation, optionally changing it with the first 
 argument if you happen to be doing the translation.
 
-=item $r->args( [$query_string] )
+=item $r->args
 
 The $r->args method will return the contents of the URI I<query
 string>.  When called in a scalar context, the entire string is
@@ -397,9 +348,6 @@ I<value> pairs are returned, i.e. it can be used like this:
    $query = $r->args;
    %in    = $r->args;
 
-$r->args can also be used to set the I<query string>. This can be useful
-when redirecting a POST request.
-
 =item $r->headers_in
 
 The $r->headers_in method will return a %hash of client request
@@ -407,10 +355,6 @@ headers.  This can be used to initialize a perl hash, or one could use
 the $r->header_in() method (described below) to retrieve a specific
 header value directly.
 
-Will return a I<HASH> reference blessed into the I<Apache::Table>
-class when called in a scalar context with no "key" argument. This
-requires I<Apache::Table>.
-
 =item $r->header_in( $header_name, [$value] )
 
 Return the value of a client header.  Can be used like this:
@@ -428,16 +372,19 @@ returned.  When called in a list context, a list of parsed I<key> =>
 I<value> pairs are returned.  *NOTE*: you can only ask for this once,
 as the entire body is read from the client.
 
-=item $r->read($buf, $bytes_to_read, [$offset])
+=item $r->read_client_block($buf, $bytes_to_read)
 
-This method is used to read data from the client, 
+Read from the entity body sent by the client.  Example of use:
+
+   $r->read_client_block($buf, $r->header_in('Content-length'));
+
+=item $r->read($buf, $bytes_to_read)
+
+This method uses read_client_block() to read data from the client, 
 looping until it gets all of C<$bytes_to_read> or a timeout happens.
 
-An offset may be specified to place the read data at some other place
-than the beginning of the string.
-
 In addition, this method sets a timeout before reading with
-C<$r-E<gt>soft_timeout>.
+C<$r-E<gt>hard_timeout>
 
 =item $r->get_remote_host
 
@@ -487,13 +434,9 @@ this variable.
 =item $c->remote_ip
 
 The dotted decimal representation of the remote client's IP address.
-This is set by the server when the connection record is created so
+This is set by then server when the connection record is created so
 is always defined.
 
-You can also set this value by providing an argument to it. This is
-helpful if your server is behind a squid accelerator proxy which adds
-a X-Forwarded-For header.
-
 =item $c->local_addr
 
 A packed SOCKADDR_IN in the same format as returned by
@@ -528,7 +471,7 @@ soon.
 
 If the configuration directive B<IdentityCheck> is set to on:  then the
 first time C<$r-E<gt>get_remote_logname> is called the server does an RFC
-1413 (ident) lookup to get the remote users system name. Generally for
+1413 (ident) lookup to get the remote user's system name. Generally for
 UNI* systems this is their login. The result is cached in C<$c-E<gt>remote_logname>
 then returned.  Subsequent calls to C<$r-E<gt>get_remote_host> return the
 cached value.
@@ -537,11 +480,10 @@ If the configuration directive B<IdentityCheck> is set to off: then
 C<$r-E<gt>get_remote_logname> does nothing and C<$c-E<gt>remote_logname> is
 always undefined.
 
-=item $c->user( [$user] )
+=item $c->user 
 
 If an authentication check was successful, the authentication handler
-caches the user name here. Sets the user name to the optional first
-argument.
+caches the user name here.
 
 =item $c->auth_type
 
@@ -552,15 +494,6 @@ C<$c-E<gt>user>, if any.
 
 Returns true if the client stopped talking to us.
 
-=item $c->fileno( [$direction] )
-
-Returns the client file descriptor. If $direction is 0, the input fd
-is returned. If $direction is not null or ommitted, the output fd is
-returned.
-
-This can be used to detect client disconnect without doing any I/O,
-e.g. using IO::Select.
-
 =back
 
 =back
@@ -578,17 +511,11 @@ Returns the value of a per-directory variable specified by the
 C<PerlSetVar> directive.
 
    # <Location /foo/bar>
-   # PerlSetVar  Key  Value
+   # SetPerlVar  Key  Value
    # </Location>
 
    my $val = $r->dir_config('Key');
 
-Keys are case-insensitive.
-
-Will return a I<HASH> reference blessed into the
-I<Apache::Table> class when called in a scalar context with no
-"key" argument. See I<Apache::Table>.
-
 =item $r->requires
 
 Returns an array reference of hash references, containing information
@@ -607,7 +534,7 @@ something else and implement your own authentication scheme.
 
 Returns a reference to the current value of the per directory
 configuration directive B<AuthName>.  The AuthName directive creates
-protection realm within the server document space. To quote RFC 1945
+protection realm within the server's document space. To quote RFC 1945
 "These realms allow the protected resources on a server to be
 partitioned into a set of protection spaces, each with its own
 authentication scheme and/or authorization database." The client uses
@@ -638,21 +565,12 @@ module provides the constants to check against.
 		      $filename);
    }
 
-=item $r->get_server_port
-
-Returns the port number on which the server is listening.
-
 =item $s = $r->server
 
 Return a reference to the server info object (blessed into the
 B<Apache::Server> package).  This is really a C<server_rec*> in
 disguise.  The following methods can be used on the server object:
 
-=item $s = Apache->server
-
-Same as above, but only available during server startup for use in
-<Perl> sections, B<PerlScript> or B<PerlModule>.
-
 =item $s->server_admin
 
 Returns the mail address of the person responsible for this server.
@@ -671,77 +589,7 @@ Returns true if this is a virtual server.
 
 =item $s->names
 
-Returns the wild-carded names for ServerAlias servers. 
-
-=item $s->dir_config( $key )
-
-Alias for Apache::dir_config.
-
-=item $s->warn
-
-Alias for Apache::warn.
-
-=item $s->log_error
-
-Alias for Apache::log_error.
-
-=item $s->uid
-
-Returns the numeric user id under which the server answers requests.
-This is the value of the User directive.
-
-=item $s->gid
-
-Returns the numeric group id under which the server answers requests.
-This is the value of the Group directive.
-
-=item $s->loglevel
-
-Returns the value of the current LogLevel. This method is added by
-the Apache::Log module, which needs to be pulled in.
-
-    use Apache::Log;
-    print "LogLevel = ", $s->loglevel;
-
-If using Perl 5.005+, the following constants are defined (but not
-exported):
-
-    Apache::Log::EMERG
-    Apache::Log::ALERT
-    Apache::Log::CRIT
-    Apache::Log::ERR
-    Apache::Log::WARNING
-    Apache::Log::NOTICE
-    Apache::Log::INFO
-    Apache::Log::DEBUG
-
-=item $r->get_handlers( $hook )
-
-Returns a reference to a list of handlers enabled for $hook. $hook is
-a string representing the phase to handle. The returned list is a list
-of references to the handler subroutines.
-
-	$list = $r->get_handlers( 'PerlHandler' );
-
-=item $r->set_handlers( $hook, [\&handler, ... ] )
-
-Sets the list if handlers to be called for $hook. $hook is a string
-representing the phase to handle. The list of handlers is an anonymous
-array of code references to the handlers to install for this request
-phase. The special list [ \&OK ] can be used to disable a particular
-phase.
-
-	$r->set_handlers( PerlLogHandler => [ \&myhandler1, \&myhandler2 ] );
-	$r->set_handlers( PerlAuthenHandler => [ \&OK ] );
-
-=item $r->push_handlers( $hook, \&handler )
-
-Pushes a new handler to be called for $hook. $hook is a string
-representing the phase to handle. The handler is a reference to a
-subroutine to install for this request phase. This handler will be
-called before any configured handlers.
-
-	$r->push_handlers( PerlHandler => \&footer);
+Returns the wild-carded names for HostAlias servers. 
 
 =back
 
@@ -784,7 +632,7 @@ for the realm defined by the configuration directive C<AuthName>.
 
 Set the handler for a request.
 Normally set by the configuration directive C<AddHandler>.
-
+  
    $r->handler( "perl-script" );
 
 =item $r->notes( $key, [$value] )
@@ -794,48 +642,9 @@ optionally set the value of a named entry.  This table is used by Apache
 modules to pass messages amongst themselves. Generally if you are
 writing handlers in mod_perl you can use Perl variables for this.
 
-   $r->notes("MY_HANDLER" => OK);
+   $r->notes("MY_HANDLER", OK);
    $val = $r->notes("MY_HANDLER");
 
-Will return a I<HASH> reference blessed into the I<Apache::Table>
-class when called in a scalar context with no "key" argument. This
-requires I<Apache::Table>.
-
-=item $r->pnotes( $key, [$value] )
-
-Like $r->notes, but takes any scalar as an value.
-
-   $r->pnotes("MY_HANDLER" => [qw(one two)]);
-   my $val = $r->pnotes("MY_HANDLER");
-   print $val->[0];     # prints "one"
-
-Advantage over just using a Perl variable is that $r->pnotes gets
-cleaned up after every request.
-
-=item $r->subprocess_env( $key, [$value] )
-
-Return the value of a named entry in the Apache C<subprocess_env>
-table, or optionally set the value of a named entry. This table is
-used by mod_include.  By setting some custom variables inside
-a perl handler it is possible to combine perl with mod_include nicely.
-If you say, e.g. in a PerlHeaderParserHandler
-
-   $r->subprocess_env(MyLanguage => "de");
-
-you can then write in your .shtml document:
-
-   <!--#if expr="$MyLanguage = en" -->
-   English
-   <!--#elif expr="$MyLanguage = de" -->
-   Deutsch
-   <!--#else -->
-   Sorry
-   <!--#endif -->
-
-Will return a I<HASH> reference blessed into the I<Apache::Table>
-class when called in a scalar context with no "key" argument. This
-requires I<Apache::Table>.
-
 =item $r->content_type( [$newval] )
 
 Get or set the content type being sent to the client.  Content types
@@ -852,11 +661,10 @@ Get or set the content encoding.  Content encodings are string like
 "gzip" or "compress".  This correspond to the "Content-Encoding"
 header in the HTTP protocol.
 
-=item $r->content_languages( [$array_ref] )
+=item $r->content_language( [$newval] )
 
-Get or set the content languages.  The content language corresponds to the
-"Content-Language" HTTP header and is an array reference containing strings
-such as "en" or "no".
+Get or set the content language.  The content language corresponds to the
+"Content-Language" HTTP header and is a string like "en" or "no".
 
 =item $r->status( $integer )
 
@@ -877,10 +685,6 @@ headers.  This can be used to initialize a perl hash, or one could use
 the $r->header_out() method (described below) to retrieve or set a specific
 header value directly.
 
-Will return a I<HASH> reference blessed into the I<Apache::Table>
-class when called in a scalar context with no "key" argument. This
-requires I<Apache::Table>.
-
 =item $r->header_out( $header, $value )
 
 Change the value of a response header, or create a new one.  You
@@ -901,10 +705,6 @@ The difference between headers_out and err_headers_out is that the
 latter are printed even on error, and persist across internal redirects
 (so the headers printed for ErrorDocument handlers will have them).
 
-Will return a I<HASH> reference blessed into the I<Apache::Table>
-class when called in a scalar context with no "key" argument. This
-requires I<Apache::Table>.
-
 =item $r->err_header_out( $header, [$value] )
 
 Change the value of an error response header, or create a new one.
@@ -921,21 +721,7 @@ and the client should be told not to cache it.
 =item $r->print( @list )
 
 This method sends data to the client with C<$r-E<gt>write_client>, but first
-sets a timeout before sending with C<$r-E<gt>soft_timeout>. This method is
-called instead of CORE::print when you use print() in your mod_perl programs.
-
-This method treats scalar references specially. If an item in @list is a
-scalar reference, it will be dereferenced before printing. This is a
-performance optimization which prevents unneeded copying of large strings,
-and it is subtly different from Perl's standard print() behavior.
-
-Example:
-
-   $foo = \"bar"; print($foo);
-
-The result is "bar", not the "SCALAR(0xDEADBEEF)" you might have expected. If
-you really want the reference to be printed out, force it into a scalar
-context by using C<print(scalar($foo))>.
+sets a timeout before sending with C<$r-E<gt>hard_timeout>.
 
 =item $r->send_fd( $filehandle )
 
@@ -946,49 +732,12 @@ like this:
   $r->send_fd(FILE);
   close(FILE);
 
-=item $r->internal_redirect( $newplace )
+=item $r->internal_redirect_handler( $newplace )
 
 Redirect to a location in the server namespace without 
 telling the client. For instance:
 
-   $r->internal_redirect("/home/sweet/home.html");
-
-=item $r->internal_redirect_handler( $newplace )
-
-Same as I<internal_redirect>, but the I<handler> from C<$r> is preserved.
-
-=item $r->custom_response($code, $uri)
-
-This method provides a hook into the B<ErrorDocument> mechanism,
-allowing you to configure a custom response for a given response
-code at request-time.
-
-Example:
-
-    use Apache::Constants ':common';
-
-    sub handler {
-        my($r) = @_;
-
-        if($things_are_ok) {
-	    return OK;
-        }
-
-        #<Location $r->uri>
-        #ErrorDocument 401 /error.html
-        #</Location>
-
-        $r->custom_response(AUTH_REQUIRED, "/error.html");
-
-        #can send a string too
-        #<Location $r->uri>
-        #ErrorDocument 401 "sorry, go away"
-        #</Location>
-
-        #$r->custom_response(AUTH_REQUIRED, "sorry, go away");
-
-        return AUTH_REQUIRED;
-    }
+   $r->internal_redirect_handler("/home/sweet/home.html");
 
 =back
 
@@ -1037,8 +786,6 @@ kill_timeout() will disarm either variety of timeout.
 
 reset_timeout() resets the timeout in progress.
 
-=item $r->post_connection($code_ref)
-
 =item $r->register_cleanup($code_ref)
 
 Register a cleanup function which is called just before $r->pool is
@@ -1049,10 +796,6 @@ destroyed.
        warn "registered cleanup called for ", $r->uri, "\n";
    });
 
-The I<post_connection> method is simply an alias for I<register_cleanup>, 
-as this method may be used to run code after the client connection is closed,
-which may not be a I<cleanup>.
-
 =back
 
 =head1 CGI SUPPORT
@@ -1062,6 +805,34 @@ type of interface.
 
 =over 4
 
+=item $r->cgi_env
+
+Return a %hash that can be used to set up a standard CGI environment.
+Typical usage would be:
+
+   %ENV = $r->cgi_env
+
+B<NOTE:> The $ENV{GATEWAY_INTERFACE} is set to C<'CGI-Perl/1.1'> so
+you can say:
+
+   if($ENV{GATEWAY_INTERFACE} =~ /^CGI-Perl/) {
+       # do mod_perl stuff
+   }
+   else {
+       # do normal CGI stuff
+   }
+
+When given a key => value pair, this will set an environment variable.
+
+   $r->cgi_env(REMOTE_GROUP => "camels");
+
+=item $r->cgi_var($key);
+
+Calls $r->cgi_env($key) in a scalar context to prevent the mistake
+of calling in a list context.
+
+   my $doc_root = $r->cgi_env('DOCUMENT_ROOT');
+
 =item $r->send_cgi_header()
 
 Take action on certain headers including I<Status:>, I<Location:> and
@@ -1070,8 +841,8 @@ $r->send_http_header().  Example of use:
 
    $r->send_cgi_header(<<EOT);
    Location: /foo/bar
-   Content-type: text/html
-
+   Content-type: text/html 
+   
    EOT
 
 =back
@@ -1096,9 +867,7 @@ Uh, oh.  Write a message to the server errorlog.
 
 =item $r->warn($message)
 
-For pre-1.3 versions of apache, this is just an alias for
-C<log_error>.  With 1.3+ versions of apache, this message will only be
-send to the error_log if B<LogLevel> is set to B<warn> or higher. 
+An alias for Apache->log_error.
 
 =back
 
@@ -1128,20 +897,6 @@ Returns true if the specified callback hook is enabled:
 
 =back
 
-=head1 GLOBAL VARIABLES
-
-=over 4
-
-=item $Apache::Server::Starting
-
-Set to true when the server is starting.
-
-=item $Apache::Server::ReStarting
-
-Set to true when the server is starting.
-
-=back
-
 =head1 SEE ALSO
 
 perl(1),
@@ -1155,9 +910,7 @@ Apache C API notes at C<http://www.apache.org/docs/>
 
 =head1 AUTHORS
 
-Perl interface to the Apache C API written by Doug MacEachern
-with contributions from Gisle Aas, Andreas Koenig, Eric Bartley, 
-Rob Hartill, Gerald Richter, Salvador Ortiz and others. 
+Gisle Aas <aas@sn.no> and Doug MacEachern <dougm@osf.org>
 
 =cut
 
