@@ -1,9 +1,13 @@
 package Apache::Registry;
 use Apache ();
 #use strict; #eval'd scripts will inherit hints
-use Apache::Constants qw(:common &OPT_EXECCGI &REDIRECT);
+use Apache::Constants qw(:common &OPT_EXECCGI);
+use FileHandle ();
+use File::Basename ();
+use Cwd ();
 
-$Apache::Registry::VERSION = '2.01';
+#$Id: Registry.pm,v 1.38 1997/10/16 23:21:47 dougm Exp $
+$Apache::Registry::VERSION = (qw$Revision: 1.38 $)[1];
 
 $Apache::Registry::Debug ||= 0;
 # 1 => log recompile in errorlog
@@ -13,22 +17,9 @@ Apache->module('Apache::Debug') if $Apache::Registry::Debug;
 
 my $Is_Win32 = $^O eq "MSWin32";
 
-unless (defined $Apache::Registry::NameWithVirtualHost) {
-    $Apache::Registry::NameWithVirtualHost = 1;
-}
-unless (defined $Apache::Registry::MarkLine) {
-    $Apache::Registry::MarkLine = 1;
-}
-
 sub handler {
     my $r = shift;
-    if(ref $r) {
-	$r->request($r);
-    }
-    else {
-	#warn "Registry args are: ($r, @_)\n";
-	$r = Apache->request;
-    }
+    Apache->request($r);
     my $filename = $r->filename;
     #local $0 = $filename; #this core dumps!?
     *0 = \$filename;
@@ -37,14 +28,15 @@ sub handler {
 	if $Debug && $Debug & 4;
 
     if (-r $filename && -s _) {
-	if (-d _) {
-	    return DECLINED; 
-	}
 	if (!($r->allow_options & OPT_EXECCGI)) {
 	    $r->log_reason("Options ExecCGI is off in this directory",
 			   $filename);
 	    return FORBIDDEN;
  	}
+	if (-d _) {
+	    $r->log_reason("attempt to invoke directory as script", $filename);
+	    return FORBIDDEN;
+	}
 	unless (-x _ or $Is_Win32) {
 	    $r->log_reason("file permissions deny server execution",
 			   $filename);
@@ -53,26 +45,23 @@ sub handler {
 
 	my $mtime = -M _;
 
-	my $uri = $r->uri;
-	$uri = "/__INDEX__" if $uri eq "/";
 	# turn into a package name
 	$r->log_error(sprintf "Apache::Registry::handler examining %s",
-		      $uri) if $Debug && $Debug & 4;
+		      $r->uri) if $Debug && $Debug & 4;
 	my $script_name = $r->path_info ?
-	    substr($uri, 0, length($uri)-length($r->path_info)) :
-		$uri;
+	    substr($r->uri, 0, length($r->uri)-length($r->path_info)) :
+		$r->uri;
 
-	if($Apache::Registry::NameWithVirtualHost) {
-	    my $name = $r->get_server_name;
-	    $script_name = join "", $name, $script_name if $name;
-	}
+	my $srv = $r->server;
+	$script_name = join "", $srv->server_hostname, $script_name
+	    if $srv->is_virtual;
 
 	# Escape everything into valid perl identifiers
 	$script_name =~ s/([^A-Za-z0-9\/])/sprintf("_%2x",unpack("C",$1))/eg;
 
 	# second pass cares for slashes and words starting with a digit
 	$script_name =~ s{
-			  (/+)       # directory
+			  (/)        # directory
 			  (\d?)      # package's first character
 			 }[
 			   "::" . ($2 ? sprintf("_%2x",unpack("C",$2)) : "")
@@ -83,78 +72,75 @@ sub handler {
 	$r->log_error("Apache::Registry::handler package $package")
 	   if $Debug && $Debug & 4;
 
-	$r->chdir_file;
+
+	my $cwd = Cwd::fastcwd();
+	chdir File::Basename::dirname($r->filename);
 
 	if (
-	    exists $Apache::Registry->{$package}{'mtime'}
+	    exists $Apache::Registry->{$package}{mtime}
 	    &&
-	    $Apache::Registry->{$package}{'mtime'} <= $mtime
+	    $Apache::Registry->{$package}{mtime} <= $mtime
 	   ){
 	    # we have compiled this subroutine already, nothing left to do
  	} else {
 	    $r->log_error("Apache::Registry::handler reading $filename")
 		if $Debug && $Debug & 4;
-	    my $sub = $r->slurp_filename;
-	    $sub = parse_cmdline($$sub);
+	    my($sub);
+	    {
+		my $fh = FileHandle->new($filename);
+		local $/;
+		$sub = <$fh>;
+		$sub = parse_cmdline($sub);
+	    }
 
 	    # compile this subroutine into the uniq package name
             $r->log_error("Apache::Registry::handler eval-ing") if $Debug && $Debug & 4;
  	    undef &{"$package\::handler"} unless $Debug && $Debug & 4; #avoid warnings
-	    if($package->can('undef_functions')) {
-		$package->undef_functions;
-	    }
 	    $r->clear_rgy_endav($script_name);
 
-	    my $line = $Apache::Registry::MarkLine ?
-		"\n#line 1 $filename\n" : "";
- 
 	    my $eval = join(
 			    '',
 			    'package ',
 			    $package,
  			    ';use Apache qw(exit);',
  			    'sub handler {',
-			    $line,
+ 			    "\n#line 1 $filename\n",
 			    $sub,
 			    "\n}", # last line comment without newline?
 			   );
 	    compile($eval);
-	    $r->stash_rgy_endav($script_name);
 	    if ($@) {
 		$r->log_error($@);
-		$@{$uri} = $@;
 		return SERVER_ERROR unless $Debug && $Debug & 2;
 		return Apache::Debug::dump($r, SERVER_ERROR);
 	    }
             $r->log_error(qq{Compiled package \"$package\" for process $$})
 	       if $Debug && $Debug & 1;
-	    $Apache::Registry->{$package}{'mtime'} = $mtime;
+	    $Apache::Registry->{$package}{mtime} = $mtime;
 	}
 
 	my $cv = \&{"$package\::handler"};
-	eval { &{$cv}($r, @_) } if $r->seqno;
-	$r->chdir_file("$Apache::Server::CWD/");
+	eval {$cv->($r, @_);} if $r->seqno;
+	{
+	    local $^W = 0; #shutup Cwd.pm
+	    chdir $cwd;
+	}
 	$^W = $oldwarn;
 
-	my $errsv = "";
-	if($@) {
-	    $errsv = $@;
-	    $@ = ''; #XXX fix me, if we don't do this Apache::exit() breaks
-	    $@{$uri} = $errsv;
+	my $err = $@;
+	if(exists $INC{'CGI.pm'}
+	   && ! $Apache::Registry::CGI_versioncheck_done ) {
+	    $Apache::Registry::CGI_versioncheck_done++;
+	    require Exporter;
+	    eval { Exporter::require_version('CGI', 2.36); }; 
+	    $err .= $@ if $@;
 	}
-
-	if($errsv) {
-	    $r->log_error($errsv);
+	if ($err) {
+	    $r->log_error($err);
 	    return SERVER_ERROR unless $Debug && $Debug & 2;
 	    return Apache::Debug::dump($r, SERVER_ERROR);
 	}
 
-#	#XXX
-#	if(my $loc = $r->header_out("Location")) {
-#	    if($r->status == 200 and substr($loc, 0, 1) ne "/") {
-#		return REDIRECT;
-#	    }
-#	}
 	return $r->status;
     } else {
 	return NOT_FOUND unless $Debug && $Debug & 2;
@@ -171,7 +157,7 @@ sub compile {
 #XXX not good enough yet
 my(%switches) = (
    'T' => sub {
-       Apache::warn("Apache::Registry: T switch ignored, ".
+       Apache::warn("Apache::Registry: -T switch ignored, ".
 		    "enable with 'PerlTaintCheck On'\n")
 	   unless $Apache::__T; "";
    },
@@ -202,8 +188,6 @@ sub parse_cmdline {
 #trick so we show up under CPAN/modules/by-module/CGI/
 package CGI::mod_perl;
 
-sub DESTROY {}
-
 1;
 
 __END__
@@ -282,15 +266,15 @@ The environment variable B<GATEWAY_INTERFACE> is set to C<CGI-Perl/1.1>.
 Normally when a Perl script is run from the command line or under CGI,
 arguments on the `#!' line are passed to the perl interpreter for processing.
 
-Apache::Registry currently only honors the B<-w> switch and will turn
+Apache::Registry currently only honors the C<-w> switch and will turn
 on warnings using the C<$^W> global variable.  Another common switch
-used with CGI scripts is B<-T> to turn on taint checking.  This can
+used with CGI scripts is C<-T> to turn on taint checking.  This can
 only be enabled when the server starts with the configuration
 directive:
 
  PerlTaintCheck On
 
-However, if taint checking is not enabled, but the B<-T> switch is seen,
+However, if taint checking is not enabled, but the C<-T> switch is seen,
 Apache::Registry will write a warning to the error_log.
 
 =head1 DEBUGGING
@@ -319,5 +303,6 @@ perl(1), mod_perl(3), Apache(3), Apache::Debug(3)
 
 =head1 AUTHORS
 
-Andreas J. Koenig and Doug MacEachern
+Andreas Koenig <andreas.koenig@franz.ww.tu-berlin.de> and
+Doug MacEachern <dougm@osf.org>
 
